#include "chatform.h"
#include "ui_chatform.h"
#include "keypressfilter.h"

ChatForm::ChatForm(QWidget *parent) :
	QWidget(parent),
	ui(new Ui::ChatForm)
{
	ui->setupUi(this);

	setFocusProxy(ui->plainTextEdit);

	mFilter = new KeyPressFilter(this);
	ui->plainTextEdit->installEventFilter(mFilter);
	connect(mFilter, SIGNAL(returnPressed()), this, SLOT(onReturnPressed()));
}

ChatForm::~ChatForm()
{
	delete ui;
}

void ChatForm::changeEvent(QEvent *e)
{
	QWidget::changeEvent(e);

	switch (e->type())
	{
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}

void ChatForm::addMessage(const QString &message)
{
	if (mDisplayName.length() == 0)
	{
		QStringList list = windowTitle().split('@');
		mDisplayName = list[0];
	}

	QString formatted = "<font color='red'><b>%1:</b></font> %2";
	ui->textEdit->append(formatted.arg(mDisplayName, Qt::escape(message)));
}

void ChatForm::onReturnPressed()
{
	QString message = Qt::escape(ui->plainTextEdit->toPlainText());
	ui->plainTextEdit->clear();

	if (message.length() > 0)
	{
		emit messageEntered(mJid, message);

		QString formatted = "<font color='blue'><b>Me:</b></font> %2";
		ui->textEdit->append(formatted.arg(message));
	}
}
