#include "xmlelement.h"

XmlElement::XmlElement()
{
}

XmlElement::XmlElement(const XmlElement& elem) :
	mTagName(elem.mTagName),
	mAttributes(elem.mAttributes),
	mData(elem.mData)
{
	if (mData.size() == 0)
	{
		for (uint i=0; i<elem.numOfChildren(); ++i)
			addChild(new XmlElement(*elem.child(i)));
	}
}

XmlElement::XmlElement(const QString& tagName) :
	mTagName(tagName)
{
}

XmlElement::~XmlElement()
{
	for (uint i=0; i<numOfChildren(); ++i)
	{
		if (mChildren[i] != NULL)
		{
			delete mChildren[i];
			mChildren[i] = NULL;
		}
	}
}

XmlElement& XmlElement::operator=(const XmlElement& elem)
{
	mTagName = elem.mTagName;
	mAttributes = elem.mAttributes;
	mData = elem.mData;
	mChildren.clear();

	if (mData.size() == 0)
	{
		for (uint i=0; i<elem.numOfChildren(); ++i)
			addChild(new XmlElement(*elem.child(i)));
	}

	return *this;
}

XmlElement* XmlElement::child(const QString& tagName) const
{
	for (uint i=0; i<numOfChildren(); ++i)
	{
		if (mChildren[i]->mTagName == tagName)
			return mChildren[i];

		XmlElement* elem = mChildren[i]->child(tagName);

		if (elem != NULL)
			return elem;
	}


	return NULL;
}

QString XmlElement::toString() const
{
	QString strXml = "<" + mTagName;

	QMap<QString, QString>::const_iterator iter = mAttributes.begin();
	while (iter != mAttributes.end())
	{
		strXml += " " + iter.key() + "=\'" + iter.value() + "\'";
		++iter;
	}

	if (mData.size() == 0 && numOfChildren() == 0)
	{
		strXml += " />";
	}
	else
	{
		strXml += ">";

		if (mData.size() > 0)
		{
			strXml += mData;
		}
		else
		{
			for (uint i = 0; i < numOfChildren(); ++i)
				strXml += mChildren[i]->toString();
		}

		strXml += "</" + mTagName + ">";
	}

	return strXml;
}
