#ifndef XMLELEMENT_H
#define XMLELEMENT_H

#include <QString>
#include <QVector>
#include <QMap>

class XmlElement
{
public:
	XmlElement();
	XmlElement(const XmlElement& elem);
	XmlElement(const QString& tagName);
	~XmlElement();

	XmlElement& operator=(const XmlElement& elem);

	void addChild(XmlElement* elem);
	XmlElement* child(uint index) const;
	XmlElement* child(const QString& tagName) const;
	uint numOfChildren() const;

	void setAttribute(const QString& name, const QString& value);
	QString attribute(const QString& name) const;
	bool hasAttribute(const QString& name) const;

	void setTagName(const QString& tagName);
	QString tagName() const;

	void setData(const QString& data);
	void appendData(const QString& data);
	QString data() const;
	void clearData();

	QString toString() const;

private:
	QString mTagName;
	QMap<QString, QString> mAttributes;
	QString mData;
	QVector<XmlElement*> mChildren;
};

// Inline functions

inline void XmlElement::addChild(XmlElement* elem)
{
	Q_ASSERT(elem != NULL);
	mChildren.push_back(elem);
}

inline XmlElement* XmlElement::child(uint index) const
{
	if (index < numOfChildren())
		return mChildren[index];

	return NULL;
}

inline uint XmlElement::numOfChildren() const
{
	return mChildren.size();
}

inline void XmlElement::setAttribute(const QString& name, const QString& value)
{
	mAttributes[name] = value;
}

inline QString XmlElement::attribute(const QString& name) const
{
	return mAttributes[name];
}

inline bool XmlElement::hasAttribute(const QString& name) const
{
	if (mAttributes.find(name) == mAttributes.end())
		return false;

	return true;
}

inline void XmlElement::setTagName(const QString& tagName)
{
	mTagName = tagName;
}

inline QString XmlElement::tagName() const
{
	return mTagName;
}

inline void XmlElement::setData(const QString& data)
{
	mData = data;
}

inline void XmlElement::appendData(const QString& data)
{
	mData += data;
}

inline QString XmlElement::data() const
{
	return mData;
}

inline void XmlElement::clearData()
{
	mData.clear();
}

#endif // XMLELEMENT_H
