#include "xmlhandler.h"
#include "xmlelement.h"

XmlHandler::XmlHandler() :
	mRootElem(true)
{
}

XmlHandler::~XmlHandler()
{
	if (mStack.size() > 0)
		delete mStack.front();
}

bool XmlHandler::startElement(const QString &namespaceURI,
							  const QString &localName,
							  const QString &qName,
							  const QXmlAttributes &atts)
{
	Q_UNUSED(qName);

	XmlElement* elem = new XmlElement(namespaceURI + ":" + localName);

	for (int i=0; i<atts.count(); ++i)
		elem->setAttribute(atts.qName(i), atts.value(i));

	if (mRootElem)
	{
		mRootElem = false;
		emit streamStarted(elem);
		delete elem;
	}
	else
	{
		if (mStack.size() > 0)
		{
			mStack.back()->addChild(elem);
			mStack.back()->clearData();
		}

		mStack.push_back(elem);
	}

	return true;
}

bool XmlHandler::endElement(const QString &namespaceURI,
							const QString &localName,
							const QString &qName)
{
	Q_UNUSED(namespaceURI);
	Q_UNUSED(localName);
	Q_UNUSED(qName);

	if (mStack.size() == 0)
	{
		emit streamEnded();
	}
	else
	{
		if (mStack.size() == 1)
		{
			emit elementReady(mStack.back());
			delete mStack.back();
		}

		mStack.pop_back();
	}

	return true;
}

bool XmlHandler::characters(const QString &ch)
{
	if (mStack.size() > 0 && mStack.back()->numOfChildren() == 0)
	{
		mStack.back()->appendData(ch);
	}

	return true;
}
