#ifndef XMPPCLIENT_H
#define XMPPCLIENT_H

#include <QString>
#include <QSslSocket>
#include <QXmlInputSource>
#include <QXmlSimpleReader>
#include "streamfeatures.h"

class XmlHandler;
class XmlElement;

class XmppClient : public QObject
{
	Q_OBJECT
public:
	XmppClient();
	~XmppClient();

	void connectToHost(const QString &username,
					   const QString &domain,
					   const QString &password,
					   const QString &hostname,
					   ushort port);
	void disconnectFromHost();

public slots:
	void sendMessage(QString to, QString message);
	void sendStatus(QString status, QString message);

private:
	void sendMessageToHost(const QString &msg);
	void processQueuedData();
	void processStreamFeatures(const XmlElement *features);
	void processSaslChallenge(const QString &challenge);
	void processIqStanza(const XmlElement *iq);
	void processMessageStanza(const XmlElement *msg);
	void processPresenceStanza(const XmlElement *presence);
	void processRoster(const XmlElement *iq);
	void selectStreamFeatures();
	QString digestMd5Response(const QString &challenge, bool auth);

private:
	QSslSocket *mSocket;
	QXmlInputSource *mXmlSource;
	QXmlSimpleReader *mXmlReader;
	XmlHandler *mXmlHandler;

	QVector<XmlElement*> mQueue;

	QString mUsername;
	QString mDomain;
	QString mPassword;
	QString mJid;

	StreamFeatures mStreamFeatures;

	bool mNewStreamInitiated;
	bool mUserAuthenticated;
	bool mSaslResponseSent;
	bool mResourceBound;
	bool mSessionEstablished;

	QString mBindId;
	QString mSessionId;
	QString mRosterId;

	QString mExpectedAuthResponse;

signals:
	void disconnected();
	void authSuccessful();
	void authFailed();
	void rosterItemReceived(QString jid, QString name);
	void statusReceived(QString from, QString status, QString message);
	void messageReceived(QString from, QString message);

private slots:
	void initiateNewStream();
	void readFromSocket();
	void onStreamStarted(const XmlElement *elem);
	void onStreamEnded();
	void queueElement(const XmlElement *elem);
};

#endif // XMPPCLIENT_H
